#include <stdio.h>
#include <stdlib.h>

void usage()
{

    puts("tm: decoder for TMDS binary data");
    puts("\tUsage:");
    puts("\t\t tm <red_bit> <green_bit> <blue_bit> <clk_bit> [<dbg_bit>]");
    puts("\t\t   where each <xxx_bit> indicates which bit of the byte");
    puts("\t\t   contains the data for channel xxx");
    puts("\t\t   and optional <dbg_bit> can be used for");
    puts("\t\t   helping identifying events like vsync etc");
        
    exit(0);
}

int terc(int val)
{
    switch(val)
    {
    case 0x29c: return 0;
    case 0x263: return 1;
    case 0x2e4: return 2;
    case 0x2e2: return 3;
    case 0x171: return 4;
    case 0x11e: return 5;
    case 0x18e: return 6;
    case 0x13c: return 7;
    case 0x2cc: return 8;
    case 0x139: return 9;
    case 0x19c: return 10;
    case 0x2c6: return 11;
    case 0x28e: return 12;
    case 0x271: return 13;
    case 0x163: return 14;
    case 0x2c3: return 15;

    }
    return -1;
}

/*
            long    %10_1001_1100 << 22   ' TERC4 0000
            long    %10_0110_0011 << 22   ' TERC4 0001
            long    %10_1110_0100 << 22   ' TERC4 0010
            long    %10_1110_0010 << 22   ' TERC4 0011
            long    %01_0111_0001 << 22   ' TERC4 0100
            long    %01_0001_1110 << 22   ' TERC4 0101
            long    %01_1000_1110 << 22   ' TERC4 0110
            long    %01_0011_1100 << 22   ' TERC4 0111
            long    %10_1100_1100 << 22   ' TERC4 1000
            long    %01_0011_1001 << 22   ' TERC4 1001
            long    %01_1001_1100 << 22   ' TERC4 1010
            long    %10_1100_0110 << 22   ' TERC4 1011
            long    %10_1000_1110 << 22   ' TERC4 1100
            long    %10_0111_0001 << 22   ' TERC4 1101
            long    %01_0110_0011 << 22   ' TERC4 1110
            long    %10_1100_0011 << 22   ' TERC4 1111

*/

int printbin(int val, int bits)
{
    int i,t;
    for (i = 0; i < bits; i++)
        printf("%d",((val >> (bits-1-i))& 1)?1:0);
    i = terc(val);
    if (i<0)
        printf("      ");
    else
        printf(" %d%d%d%d ",i&8 ? 1:0, i&4 ? 1:0, i&2 ? 1:0, i&1 ? 1:0);
    return (bits);
}

void display(int red, int green, int blue, int clkbits, int dbgbits, int bits)
{
    // show each in binary
    printf("%c %2d\t ---> \t", bits == 10 ? ' ':'*', bits);
    printbin(red, bits);
    printf("\t");
    printbin(green, bits);
    printf("\t");
    printbin(blue, bits);
    printf("\t\t");
    printbin(clkbits, bits);
    printf("\t");
    printbin(dbgbits, bits);
    printf("\t\n");
}

int main(int argc, char **argv)
{
    int red, green, blue, clk;
    int r = 0, g = 0, b = 0;
    int ch;
    int oldclk = 0;
    int newclk;
    int count = 0;
    int bits = 0;
    int clkbits = 0;
    int dbgbits = 0;
    int dbgpin = 0;

    if (argc<5)
        usage();
    
    r = atoi(argv[1]);
    g = atoi(argv[2]);
    b = atoi(argv[3]);
    clk = atoi(argv[4]);
    if (argc > 5)
        dbgpin = atoi(argv[5]);

    printf("Decoding TMDS bits on binary input bytes:\n");
    printf("  sample    bits \tRed channel 2\t\tGreen channel 1\t\tBlue channel 0\t\t\tClock \t\t\tDebug\n");
    //printf("red=%d green=%d blue=%d clk=%d\n",r,g,b,clk); 
    while (1)
    {
        ch=getchar();
        if (ch == EOF)
        {
            //printf("End of file detected at pos %d\n",count);
            break;
        }

        newclk = (ch >> clk) & 1;
        if ((oldclk == 0) && newclk) 
        { // new sample starts
            // display previous sample
            count++;
            printf("%10d ",count);
            display(red,green,blue,clkbits, dbgbits, bits);
            red=green=blue=bits=clkbits=dbgbits=0;   
        } 
        red     |= ((ch >> r)& 1) ? (1<<bits): 0;
        green   |= ((ch >> g)& 1) ? (1<<bits): 0;
        blue    |= ((ch >> b)& 1) ? (1<<bits): 0;
        clkbits |= ((ch >> clk) & 1) ? (1<<bits): 0;
        dbgbits |= ((ch >> dbgpin) & 1) ? (1<<bits): 0;
        oldclk = newclk;
        bits++;
    }
    puts("Done");
    return 0;
}


